@@ -63,19 +63,20 @@ NULL
 #' @rdname format
 #' @export
 format.datetimeoffset <- function(x, ...) {
-    x <- update_nas(x)
+    x <- update_nas(x, allow_times = TRUE)
     s <- purrr::map_chr(x, format_helper)
-    is.na(s) <- is.na(field(x, "year"))
+    is.na(s) <- is.na(field(x, "year")) & !is_time(x)
     s
 }
 
 #' @rdname format
 #' @param sep UTC offset separator.  Either ":" or "".
-#' @param mode  If `mode = "xmp"` only output valid XMP metadata datetime values.
-#'              If `mode = "pdf"` only output supported PDF docinfo datetime values.
+#' @param mode  If `mode = "pdf"` only output supported PDF docinfo datetime values.
+#'              If `mode = "toml"` only output supported TOML datetime values.
+#'              If `mode = "xmp"` only output valid XMP metadata datetime values.
 #' @export
 format_iso8601 <- function(x, offsets = TRUE, precision = NULL, sep = ":",
-                           mode = c("normal", "xmp"), ...) {
+                           mode = c("normal", "toml", "xmp"), ...) {
     mode <- match.arg(mode)
     purrr::map_chr(x, format_iso8601_helper, offsets = offsets, precision = precision, sep = sep, mode = mode)
 }
@@ -194,20 +195,21 @@ format_iso8601_helper <- function(x, offsets = TRUE, precision = NULL, sep = ":"
                                   mode = "normal", ...) {
     if (is.na(x)) return(NA_character_)
 
-    precision <- precision %||% datetime_precision(x)
+    precision <- precision %||%
+        ifelse(is_time(x), datetime_precision(x, unspecified = TRUE), datetime_precision(x))
     stopifnot(precision %in% c("year", "month", "day", "hour", "minute", "second",
                                "decisecond", "centisecond", "millisecond",
                                "hundred microseconds", "ten microseconds", "microsecond",
                                "hundred nanoseconds", "ten nanoseconds", "nanosecond"))
     stopifnot(sep %in% c(":", ""))
     x <- datetime_narrow(x, precision)
-    if (isFALSE(offsets)) {
+    if (isFALSE(offsets) || (is_time(x) && mode == "toml")) {
         x <- set_hour_offset(x, NA_integer_)
         x <- set_minute_offset(x, NA_integer_)
         x <- set_tz(x, NA_character_)
     }
-    x <- update_nas(x)
-    if (mode == "xmp") {
+    x <- update_nas(x, allow_times = TRUE, toml = (mode == "toml"))
+    if (mode %in% c("toml", "xmp")) {
         if (is.na(field(x, "minute")) && !is.na(field(x, "hour")))
             field(x, "minute") <- 0L
         if (is.na(field(x, "minute_offset")) && !is.na(field(x, "hour_offset")))
@@ -216,7 +218,10 @@ format_iso8601_helper <- function(x, offsets = TRUE, precision = NULL, sep = ":"
     year_str <- my_format_year(field(x, "year"))
     month_str <- my_format(field(x, "month"), prefix = "-")
     day_str <- my_format(field(x, "day"), prefix = "-")
-    hour_str <- my_format(field(x, "hour"), prefix = "T")
+    if (is_time(x) && mode == "toml")
+        hour_str <- my_format(field(x, "hour"), prefix = "")
+    else
+        hour_str <- my_format(field(x, "hour"), prefix = "T")
     minute_str <- my_format(field(x, "minute"), prefix = ":")
     second_str <- my_format(field(x, "second"), prefix = ":")
     nanosecond_str <- my_format_nanosecond(field(x, "nanosecond"), field(x, "subsecond_digits"))
@@ -224,6 +229,8 @@ format_iso8601_helper <- function(x, offsets = TRUE, precision = NULL, sep = ":"
     s <- paste0(year_str, month_str, day_str,
                 hour_str, minute_str, second_str, nanosecond_str,
                 offset_str)
+    if (s == "")
+        s <- NA_character_
     s
 }
 
@@ -287,21 +294,38 @@ format_edtf_helper <- function(x, offsets, precision, usetz) {
     minute_str <- my_format(field(x, "minute"), prefix = ":", edtf = TRUE, blank = precision < PRECISION_MINUTE)
     second_str <- my_format(field(x, "second"), prefix = ":", edtf = TRUE, blank = precision < PRECISION_SECOND)
     nanosecond_str <- my_format_nanosecond(field(x, "nanosecond"), field(x, "subsecond_digits"),
-                                           edtf = TRUE, blank = precision < PRECISION_NANOSECOND)
+                                           edtf = TRUE, blank = precision <= PRECISION_SECOND)
     offset_str <- my_format_tz(x, edtf = offsets, add_tz = usetz)
     paste0(year_str, month_str, day_str,
            hour_str, minute_str, second_str, nanosecond_str,
            offset_str)
 }
 
-update_nas <- function(x, pdfmark = FALSE) {
-    if (pdfmark) { # pdfmark years in between 0 and 9999
+is_time <- function(x) {
+    is.na(get_year(x)) & is.na(get_month(x)) & is.na(get_day(x)) & !is.na(get_hour(x))
+}
+
+update_nas <- function(x, allow_times = FALSE, pdfmark = FALSE, toml = FALSE) {
+    if (pdfmark || toml) { # pdfmark years in between 0 and 9999
         year <- get_year(x)
         is.na(x) <- is.na(x) | (year < 0L) | (year > 9999L)
     }
+    # TOML doesn't allow YYYY or YYYY-MM dates as well as HH or HH:MM times
+    # If HH:MM we'll assume 00 seconds
+    # Other three cases we'll coerce to NA_datetimeoffset_
+    if (toml) {
+        x <- set_month(x, ifelse(is.na(get_day(x)), NA_integer_, get_month(x)))
+        x <- set_year(x, ifelse(is.na(get_month(x)), NA_integer_, get_year(x)))
+        x <- set_hour(x, ifelse(is.na(get_minute(x)), NA_integer_, get_hour(x)))
+        x <- set_second(x, ifelse(is.na(get_second(x)), 0L, get_second(x)))
+    }
     # No smaller time units if missing bigger time units
+    x <- set_month(x, ifelse(is.na(get_year(x)), NA_integer_, get_month(x)))
     x <- set_day(x, ifelse(is.na(get_month(x)), NA_integer_, get_day(x)))
-    x <- set_hour(x, ifelse(is.na(get_day(x)), NA_integer_, get_hour(x)))
+    if (allow_times)
+        x <- set_hour(x, ifelse(!is_time(x) & is.na(get_day(x)), NA_integer_, get_hour(x)))
+    else
+        x <- set_hour(x, ifelse(is.na(get_day(x)), NA_integer_, get_hour(x)))
     x <- set_minute(x, ifelse(is.na(get_hour(x)), NA_integer_, get_minute(x)))
     x <- set_second(x, ifelse(is.na(get_minute(x)), NA_integer_, get_second(x)))
     x <- set_nanosecond(x, ifelse(is.na(get_second(x)), NA_integer_, get_nanosecond(x)))
